@@ -75,6 +75,7 @@ Functions doc
 import os.path
 import logging
 from collections.abc import Mapping
+import copy
 
 from .io_ctrl import single_input, input_control, output_register_callback, send_msg, single_input_kwargs
 from .session import get_current_session, get_current_task_id
@@ -299,6 +300,7 @@ def _parse_select_options(options):
     # value (label same as value)
     opts_res = []
     for opt in options:
+        opt = copy.deepcopy(opt)
         if isinstance(opt, Mapping):
             assert 'value' in opt and 'label' in opt, 'options item must have value and label key'
         elif isinstance(opt, (list, tuple)):
@@ -306,7 +308,6 @@ def _parse_select_options(options):
             opt = dict(zip(('label', 'value', 'selected', 'disabled'), opt))
         else:
             opt = dict(value=opt, label=opt)
-        opt['value'] = opt['value']
         opts_res.append(opt)
 
     return opts_res
@@ -408,6 +409,10 @@ def radio(label='', options=None, *, inline=None, validate=None, name=None, valu
     if value is not None:
         del item_spec['value']
         item_spec['options'] = _set_options_selected(item_spec['options'], value)
+
+    # From https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/required
+    # In the case of a same named group of radio buttons, if a single radio button in the group has the required attribute,
+    # a radio button in that group must be checked, although it doesn't have to be the one with the attribute is applied
     if required is not None:
         del item_spec['required']
         item_spec['options'][-1]['required'] = required
@@ -430,6 +435,7 @@ def _parse_action_buttons(buttons):
     """
     act_res = []
     for act in buttons:
+        act = copy.deepcopy(act)
         if isinstance(act, Mapping):
             assert 'label' in act, 'actions item must have label key'
             assert 'value' in act or act.get('type', 'submit') != 'submit' or act.get('disabled'), \
@@ -462,7 +468,8 @@ def actions(label='', buttons=None, name=None, help_text=None):
                 "label":(str) button label,
                 "value":(object) button value,
                 "type":(str, optional) button type,
-                "disabled":(bool, optional) whether the button is disabled
+                "disabled":(bool, optional) whether the button is disabled,
+                "color":(str, optional) button color
              }
 
           When ``type='reset'/'cancel'`` or ``disabled=True``, ``value`` can be omitted
@@ -483,6 +490,9 @@ def actions(label='', buttons=None, name=None, help_text=None):
           Note: After clicking the ``type=reset`` button, the form will not be submitted,
           and the ``actions()`` call will not return
 
+        The ``color`` of button can be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`,
+        `dark`.
+
     :param - label, name, help_text: Those arguments have the same meaning as for `input()`
     :return: If the user clicks the ``type=submit`` button to submit the form,
         return the value of the button clicked by the user.
@@ -491,7 +501,7 @@ def actions(label='', buttons=None, name=None, help_text=None):
     When ``actions()`` is used as the last input item in `input_group()` and contains a button with ``type='submit'``,
     the default submit button of the `input_group()` form will be replace with the current ``actions()``
 
-    **usage scenes of ``actions()``**
+    **usage scenes of actions() **
 
     .. _custom_form_ctrl_btn:
 
@@ -523,8 +533,8 @@ def actions(label='', buttons=None, name=None, help_text=None):
             actions('actions', [
                 {'label': 'Save', 'value': 'save'},
                 {'label': 'Save and add next', 'value': 'save_and_continue'},
-                {'label': 'Reset', 'type': 'reset'},
-                {'label': 'Cancel', 'type': 'cancel'},
+                {'label': 'Reset', 'type': 'reset', 'color': 'warning'},
+                {'label': 'Cancel', 'type': 'cancel', 'color': 'danger'},
             ], name='action', help_text='actions'),
         ])
         put_code('info = ' + json.dumps(info, indent=4))
@@ -590,6 +600,18 @@ def file_upload(label='', accept=None, name=None, placeholder='Choose file', mul
         :func:`path_deploy() <pywebio.platform.path_deploy>` to start the PyWebIO application,
         the maximum file size to be uploaded allowed by the web framework can be set through the ``max_payload_size`` parameter.
 
+    .. exportable-codeblock::
+        :name: file_upload_example
+        :summary: `file_upload()` example
+
+        # Upload a file and save to server                      # ..doc-only
+        f = input.file_upload("Upload a file")                  # ..doc-only
+        open('asset/'+f['filename'], 'wb').write(f['content'])  # ..doc-only
+
+        imgs = file_upload("Select some pictures:", accept="image/*", multiple=True)
+        for img in imgs:
+            put_image(img['content'])
+
     """
     item_spec, valid_func, onchange_func = _parse_args(locals())
     item_spec['type'] = 'file'
