@@ -49,8 +49,8 @@ def _check_origin(origin, allowed_origins, handler: WebSocketHandler):
         return True
 
     return any(
-        fnmatch.fnmatch(origin, patten)
-        for patten in allowed_origins
+        fnmatch.fnmatch(origin, pattern)
+        for pattern in allowed_origins
     )
 
 
@@ -308,7 +308,7 @@ def start_server(applications, port=0, host='',
        Can also use a string to directly set the url of PyWebIO static resources.
     :param str static_dir: The directory to store the application static files.
        The files in this directory can be accessed via ``http://<host>:<port>/static/files``.
-       For example, if there is a ``A/B.jpg`` file in ``http_static_dir`` path,
+       For example, if there is a ``A/B.jpg`` file in ``static_dir`` path,
        it can be accessed via ``http://<host>:<port>/static/A/B.jpg``.
     :param bool remote_access: Whether to enable remote access, when enabled,
        you can get a temporary public network access address for the current application,
@@ -347,6 +347,11 @@ def start_server(applications, port=0, host='',
 
     utils.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
 
+    debug = Session.debug = os.environ.get('PYWEBIO_DEBUG', debug)
+
+    # Since some cloud server may close idle connections (such as heroku),
+    # use `websocket_ping_interval` to  keep the connection alive
+    tornado_app_settings.setdefault('websocket_ping_interval', 30)
     tornado_app_settings.setdefault('websocket_max_message_size', max_payload_size)  # Backward compatible
     tornado_app_settings['websocket_max_message_size'] = parse_file_size(
         tornado_app_settings['websocket_max_message_size'])
@@ -432,7 +437,8 @@ def start_server_in_current_thread_session():
         server.stop()
         logger.debug('Closing tornado ioloop...')
         tasks = [t for t in asyncio.all_tasks() if t is not asyncio.current_task() and not t.done()]
-        for task in tasks: task.cancel()
+        for task in tasks:
+            task.cancel()
 
         # 必须需要 await asyncio.sleep ，否则上方 task.cancel() 调用无法调度生效
         # This line must be required, otherwise the `task.cancel()` call cannot be scheduled to take effect
